#include <stdio.h>
#include <stdlib.h>
#include <gccore.h>
#include <wiiuse/wpad.h>
#include <unistd.h>
#include <jpeg/jpgogc.h>


static void *xfb = NULL;
static GXRModeObj *rmode = NULL;

ir_t ir;
int errorcount;


//	---------------------------------------------------------------------------------
int main(int argc, char **argv) {
//	---------------------------------------------------------------------------------
	// Initialise the video system
	VIDEO_Init();
	
	// This function initialises the attached controllers
	WPAD_Init();
	
	//Sets the screensize for the wiimote and the wiimote IR 
	WPAD_SetVRes(0,640,480);
	WPAD_SetDataFormat(WPAD_CHAN_0,WPAD_FMT_BTNS_ACC_IR);
	
	// Obtain the preferred video mode from the system
	// This will correspond to the settings in the Wii menu
	rmode = VIDEO_GetPreferredMode(NULL);

	// Allocate memory for the display in the uncached region
	xfb = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
	
	// Initialise the console, required for printf
	console_init(xfb,20,20,rmode->fbWidth,rmode->xfbHeight,rmode->fbWidth*VI_DISPLAY_PIX_SZ);
	
	// Set up the video registers with the chosen mode
	VIDEO_Configure(rmode);
	
	// Tell the video hardware where our display memory is
	VIDEO_SetNextFramebuffer(xfb);
	
	// Make the display visible
	VIDEO_SetBlack(FALSE);

	// Flush the video register changes to the hardware
	VIDEO_Flush();

	// Wait for Video setup to complete
	VIDEO_WaitVSync();
	if(rmode->viTVMode&VI_NON_INTERLACE) VIDEO_WaitVSync();

//======================================================================================================
//coding for Drawbox Horizontal
void DrawHLine (int x1, int x2, int y, int color) {
	int i;
	y = 320 * y;
	x1 >>= 1;
	x2 >>= 1;
	for (i = x1; i <= x2; i++) {
		u32 *tmpfb = xfb;
		tmpfb[y+i] = color;
	}
}
//coding for Drawbox Vertical
void DrawVLine (int x, int y1, int y2, int color) {
	int i;
	x >>= 1;
	for (i = y1; i <= y2; i++) {
		u32 *tmpfb = xfb;
		tmpfb[x + (640 * i) / 2] = color;
	}
}
//finished Drawbox coding
void DrawBox (int x1, int y1, int x2, int y2, int color) {
	DrawHLine (x1, x2, y1, color);
	DrawHLine (x1, x2, y2, color);
	DrawVLine (x1, y1, y2, color);
	DrawVLine (x2, y1, y2, color);
}

//============================================================================================================	


	//-------------------------------------
	
	// The console understands VT terminal escape codes
	// This positions the cursor on row 2, column 0
	// we can use variables for this with format codes too
	// e.g. printf ("\x1b[%d;%dH", row, column );
	printf("\x1b[2;0H");
	
//	------------------------------------------------------------------------------------
	while(1) {
//	------------------------------------------------------------------------------------


		// Call WPAD_ScanPads each loop, this reads the latest controller states
		WPAD_ScanPads();
		//reads the IR variable and relays to where on the screen the cursor is.
		WPAD_IR(0, &ir);
		
		//These Variables are used to determain the coordinates of the cursor.
		int cursorx = ir.x;
		int cursory = ir.y;
		
		u16 pressed = WPAD_ButtonsDown(0);
		
		

//if (cursorx >= 320 && cursorx <=340 && cursory >= 240 && cursory <=260){

		//===================================================================================================
		//                                     Introduction!
		//===================================================================================================
		if (pressed & WPAD_BUTTON_HOME) {
			exit(0);
		}
		int xyshow;
		errorcount = 1;
		xyshow = 1;
		
		VIDEO_ClearFrameBuffer(rmode,xfb,COLOR_BLACK);
		
		//use these to show the current location of the cursor.(the first one restarts the position of the print
		//as to avoid word wraping the screen.
		printf("\x1b[3;5H");			
				//printf ("x=%d y=%d", cursorx,cursory);
		printf("Please point the cursor in the box to begin!\n");
		printf("\x1b[4;5H");
		printf("Press HOME to go back to the loader.\n");
		//printf("press 2 to show the the coordinates.");
		
		
		DrawBox (320,240,340,260,COLOR_PURPLE);
		
		//Drawbox function will make a simple cursor box.
		DrawBox (ir.x,ir.y,ir.x + 1,ir.y + 1,COLOR_WHITE);
	
		if (cursorx >= 320 && cursorx <=340 && cursory >= 240 && cursory <=260){
			while(1) {
				//===========================================================================================
				//                               LEVEL 1
				//===========================================================================================			
				WPAD_ScanPads();
				WPAD_IR(0, &ir);
				
				u16 pressed = WPAD_ButtonsDown(0);
				int cursorx = ir.x;
				int cursory = ir.y;

				
				printf("\x1b[3;3H");
				if (xyshow == 1) {
					if (pressed & WPAD_BUTTON_2) {
						xyshow ++;
					}
				}
				
				if (xyshow == 2) {
					if (pressed & WPAD_BUTTON_2) {
						xyshow --;
					}
				}
		
				if (xyshow == 2) {
					printf("\x1b[3;3H");	
					printf ("x=%d y=%d", cursorx,cursory);
				}
				
				if ( pressed & WPAD_BUTTON_HOME ) {
					exit(0);
				}
				//============================================================================================
				VIDEO_ClearFrameBuffer(rmode,xfb,COLOR_BLACK);
				//Starting pad
				DrawHLine (300,360,240,COLOR_WHITE);
				DrawVLine (300,240,280,COLOR_WHITE);
				DrawVLine (360,240,280,COLOR_WHITE);
				DrawHLine (300,320,280,COLOR_WHITE);
				DrawHLine (340,360,280,COLOR_WHITE);
				//halway 1
				DrawVLine (320,280,340,COLOR_WHITE);
				DrawVLine (340,280,360,COLOR_WHITE);
				//hallway 2
				DrawBox (210,340,230,360,COLOR_PURPLE);
				DrawHLine (210,320,340,COLOR_WHITE);
				DrawHLine (210,340,360,COLOR_WHITE);
				DrawVLine (210,340,360,COLOR_WHITE);
				//-.-.-.-.-.-.-.-.-.-.-.-.-.-BOUNDRIES-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
				if (cursorx >= 280 && cursorx <=380 && cursory >= 200 && cursory <=239){//top
						VIDEO_ClearFrameBuffer(rmode,xfb,COLOR_BLACK);
						printf("you touched the edge");
						errorcount ++;
						usleep(5000000);
				}
				if (cursorx >= 280 && cursorx <=299 && cursory >= 210 && cursory <=300){//left landingpad
						VIDEO_ClearFrameBuffer(rmode,xfb,COLOR_BLACK);
						printf("you touched the edge");
						errorcount ++;
						usleep(5000000);
				}	
				if (cursorx >= 361 && cursorx <=380 && cursory >= 210 && cursory <=300){//rightlandingpad
						VIDEO_ClearFrameBuffer(rmode,xfb,COLOR_BLACK);
						printf("you touched the edge");
						errorcount ++;
						usleep(5000000);
				}			
				if (cursorx >= 200&& cursorx <=319 && cursory >= 281 && cursory <=339){//lefthall
						VIDEO_ClearFrameBuffer(rmode,xfb,COLOR_BLACK);
						printf("you touched the edge");
						errorcount ++;
						usleep(5000000);
				}					
				if (cursorx >= 341 && cursorx <=380 && cursory >= 281 && cursory <=380){//righthall
						VIDEO_ClearFrameBuffer(rmode,xfb,COLOR_BLACK);
						printf("you touched the edge");
						errorcount ++;
						usleep(5000000);
				}
				if (cursorx >= 200 && cursorx <=380 && cursory >= 361 && cursory <=380){//bottom
						VIDEO_ClearFrameBuffer(rmode,xfb,COLOR_BLACK);
						printf("you touched the edge");
						errorcount ++ ;
						usleep(5000000);
				}	
				if (cursorx >= 210 && cursorx <=230 && cursory >= 340 && cursory <=360){//GOAL!!!!
						VIDEO_ClearFrameBuffer(rmode,xfb,COLOR_BLACK);
						errorcount -- ;
						printf("YOU WIN! you hit the walls %i times. \n", errorcount);
						printf("\x1b[4;3H");
						printf("More levels coming Soon Stay tuned to your Homebrew browser for updates!");
						usleep(900000000);
						exit(0);
						
				}					
				//-.-.-.-.-.-.-.-.-.-.-.-.-.-END BOUNDRIES-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
				//cursor
				DrawBox (ir.x,ir.y,ir.x + 1,ir.y + 1,COLOR_WHITE);
			
				VIDEO_WaitVSync();
				}
			}
		
		// Wait for the next frame
		VIDEO_WaitVSync();
	}

	return 0;
}
